#!/usr/bin/env python3


import os
import urllib.parse
import re
import sys
import subprocess
import datetime

def prompt_user():
	"""Prompt user for paths and file format"""
	# Default paths
	default_noteplan = "/Users/bertkruisdijk/Library/Containers/co.noteplan.NotePlan-setapp/Data/Library/Application Support/co.noteplan.NotePlan-setapp/Notes"
	default_cloud = "/Users/bertkruisdijk/Library/CloudStorage/Dropbox/Reference_folders"
	
	# Simple command-line dialogs using osascript (AppleScript) for Mac native feel
	cmd = ['osascript', '-e', f'text returned of (display dialog "Path to NotePlan Notes folder:" default answer "{default_noteplan}" buttons {{"Cancel", "Continue"}} default button "Continue")']
	try:
		noteplan_path = subprocess.check_output(cmd).decode('utf-8').strip()
	except subprocess.CalledProcessError:
		print("Cancelled by user")
		sys.exit(0)
		
	cmd = ['osascript', '-e', f'text returned of (display dialog "Path to reference files folder:" default answer "{default_cloud}" buttons {{"Cancel", "Continue"}} default button "Continue")']
	try:
		cloud_path = subprocess.check_output(cmd).decode('utf-8').strip()
	except subprocess.CalledProcessError:
		print("Cancelled by user")
		sys.exit(0)
		
	cmd = ['osascript', '-e', 'button returned of (display dialog "Choose file format for link notes:" buttons {".txt", ".md"} default button ".md")']
	try:
		file_format = subprocess.check_output(cmd).decode('utf-8').strip()
	except subprocess.CalledProcessError:
		print("Cancelled by user")
		sys.exit(0)
	
	# Ask if user wants to create missing folders
	cmd = ['osascript', '-e', 'button returned of (display dialog "Create folders that don\'t exist?" buttons {"No", "Yes"} default button "Yes")']
	try:
		create_folders = subprocess.check_output(cmd).decode('utf-8').strip() == "Yes"
	except subprocess.CalledProcessError:
		print("Cancelled by user")
		sys.exit(0)
		
	# Remove trailing slashes if present
	noteplan_path = noteplan_path.rstrip('/')
	cloud_path = cloud_path.rstrip('/')
	
	return noteplan_path, cloud_path, file_format, create_folders

def show_progress(current, total, message=""):
	"""Show progress dialog"""
	progress_percent = int((current / total) * 100)
	cmd = ['osascript', '-e', f'tell application "System Events" to set progress total steps to {total}',
			'-e', f'tell application "System Events" to set progress completed steps to {current}',
			'-e', f'tell application "System Events" to set progress description to "Creating folder links..."',
			'-e', f'tell application "System Events" to set progress additional description to "{message}"']
	subprocess.run(cmd, stderr=subprocess.DEVNULL)
	
def create_folder_link_notes(noteplan_root, cloud_root, file_extension, create_folders=False):
	"""Create link notes for each matching folder"""
	processed_count = 0
	diagnostic_info = []
	
	# Get all NotePlan folders
	all_folders = []
	for root, dirs, files in os.walk(noteplan_root):
		# Skip attachment folders
		if root.endswith("_attachments") or "/_attachments/" in root:
			continue
		all_folders.append(root)
		
	# Sort folders for consistent processing
	all_folders.sort()
	total_folders = len(all_folders)
	
	# Process each folder
	for i, noteplan_folder_path in enumerate(all_folders):
		folder_name = os.path.basename(noteplan_folder_path)
		
		# Skip attachment folders (additional check)
		if folder_name.endswith("_attachments"):
			continue
			
		show_progress(i+1, total_folders, f"Processing: {folder_name}")
		
		# Get relative path
		relative_path = os.path.relpath(noteplan_folder_path, noteplan_root)
		cloud_folder_path = os.path.join(cloud_root, relative_path) if relative_path != "." else cloud_root
		
		# Check if matching cloud folder exists
		if not os.path.exists(cloud_folder_path):
			if create_folders:
				try:
					os.makedirs(cloud_folder_path)
					diagnostic_info.append(f"Created new folder: {cloud_folder_path}")
				except Exception as e:
					diagnostic_info.append(f"Error creating folder {cloud_folder_path}: {e}")
					continue  # Skip this folder if we couldn't create it
			else:
				diagnostic_info.append(f"Skipped non-existent folder (creation disabled): {cloud_folder_path}")
				continue  # Skip this folder since it doesn't exist and creation is disabled
		elif not os.path.isdir(cloud_folder_path):
			diagnostic_info.append(f"Path exists but is not a directory: {cloud_folder_path}")
			continue  # Skip this folder if it's not a directory
		
		# Extract number prefix if present
		number_part = ""
		if " " in folder_name:
			first_part = folder_name.split(" ")[0]
			if first_part.replace(".", "").isdigit():
				number_part = first_part
				
		# Get contents of cloud folder
		entries = []
		for entry in os.listdir(cloud_folder_path):
			if not entry.startswith('.') and not entry.endswith('_attachments'):
				entry_path = os.path.join(cloud_folder_path, entry)
				is_folder = os.path.isdir(entry_path)
				entries.append({
					'name': entry,
					'path': entry_path,
					'is_folder': is_folder
				})
				
		# Sort entries - folders first, then files, both alphabetically
		entries.sort(key=lambda x: (0 if x['is_folder'] else 1, x['name'].lower()))
		
		# Create markdown content
		note_filename = f"_Folder_Link{file_extension}"
		note_file_path = os.path.join(noteplan_folder_path, note_filename)
		
		# Construct file URI
		cloud_folder_abs_path = os.path.abspath(cloud_folder_path)
		file_uri_path = 'file:///' + urllib.parse.quote(cloud_folder_abs_path)
		
		# Create note content with simplified title
		link_text = folder_name if relative_path == "." else relative_path
		note_content = f"# 💾 Link to Files: {number_part}\n\n"
		note_content += f"### 👉 [{link_text}]({file_uri_path})\n\n"
		
		# Add links to files and subfolders
		if entries:
			note_content += "## Contents:\n\n"
			for entry in entries:
				entry_uri = 'file:///' + urllib.parse.quote(entry['path'])
				icon = '📁' if entry['is_folder'] else '📄'
				entry_text = f"{icon} {entry['name']}"
				note_content += f"- [{entry_text}]({entry_uri})\n"
		else:
			note_content += "This folder is empty.\n"
			
		# Write the note file
		try:
			with open(note_file_path, "w") as note_file:
				note_file.write(note_content)
			processed_count += 1
		except Exception as e:
			diagnostic_info.append(f"Error writing {note_file_path}: {e}")
			
	return processed_count, diagnostic_info

def generate_report(noteplan_path, cloud_path, processed_count, diagnostic_info):
	"""Generate a report file on the desktop"""
	desktop_path = os.path.join(os.path.expanduser("~"), "Desktop")
	report_path = os.path.join(desktop_path, "FolderLinkingReport.md")
	
	report_content = "# Folder Linking Report\n\n"
	report_content += f"Date: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n\n"
	report_content += f"NotePlan Path: `{noteplan_path}`\n\n"
	report_content += f"Reference Files Path: `{cloud_path}`\n\n"
	report_content += f"Files Created: {processed_count}\n\n"
	
	if diagnostic_info:
		report_content += "## Issues:\n\n"
		for info in diagnostic_info:
			report_content += f"- {info}\n"
			
	with open(report_path, "w") as f:
		f.write(report_content)
		
	return report_path

def show_completion(processed_count, report_path):
	"""Show completion dialog with option to open report"""
	cmd = ['osascript', '-e', 
			f'set completionMsg to "Completed! Created {processed_count} folder link files." & return & return & "A report has been saved to your Desktop."',
			'-e', 'set theButton to button returned of (display dialog completionMsg buttons {"Open Report", "OK"} default button "OK")',
			'-e', 'if theButton is "Open Report" then',
			'-e', f'    do shell script "open \\"{report_path}\\""',
			'-e', 'end if']
	subprocess.run(cmd)
	
def main():
	# Get user input
	noteplan_path, cloud_path, file_format, create_folders = prompt_user()
	
	# Validate paths
	if not os.path.exists(noteplan_path) or not os.path.isdir(noteplan_path):
		cmd = ['osascript', '-e', f'display dialog "The NotePlan folder path does not exist: {noteplan_path}" buttons {{"OK"}} default button "OK"']
		subprocess.run(cmd)
		return
	
	if not os.path.exists(cloud_path) or not os.path.isdir(cloud_path):
		cmd = ['osascript', '-e', f'display dialog "The reference files folder path does not exist: {cloud_path}" buttons {{"OK"}} default button "OK"']
		subprocess.run(cmd)
		return
	
	# Process folders
	processed_count, diagnostic_info = create_folder_link_notes(noteplan_path, cloud_path, file_format, create_folders)
	
	# Generate and show report
	report_path = generate_report(noteplan_path, cloud_path, processed_count, diagnostic_info)
	show_completion(processed_count, report_path)
	
if __name__ == "__main__":
	main()